
# (C) Copyright 1996 University of Toronto
# Permission to use this software for non-commerical purposes is hereby
# granted.  This software is provided "as is" without express or implied
# warranty.  This software may not be transferred or sublicensed.
# Any enhancements made by the recipient of this software shall be
# shared with the original copyright holders.

# Acknowledgements
# This software was developed as part of the project entitled "Models
# for Supporting the Analysis and Redesign of Business Processes" funded  
# by the Information Technology Research Centre of Ontario.
# Principal investigators for the project are John Mylopoulos, Ray
# Reiter, Hector Levesque, Eric Yu, and Yves Lesperance.
# The i* modelling framework (including the Strategic Dependency and
# Strategic Rationale models) was developed by Eric Yu in his doctoral
# dissertation, Department of Computer Science, University of Toronto,
# 1995.
# This software was originally written by Chen Wang as part of his
# Master degree requirement.
# Substantial modifications and enhancements were made by Godfrey Cheng
# and Seyil Yoon, who worked as summer students on this project.
# The "repository" part of the code was originally written by Brian Kramer and
# subsequently modified by Chen Wang, Seyil Yoon, and Godfrey Cheng.


#######################################################################
# Simple Math Constants
#######################################################################

set PI 3.1415926535

#######################################################################
# Display Tag Orientation
#######################################################################

set Display_Tag(0) 0

#######################################################################
# Tag To Object
#######################################################################

set SR_Object(0) 0

#######################################################################
# Context
#######################################################################

set SR_Context(0) ""

#######################################################################
# Some stuff about colors
#######################################################################

if {[info exists env(CONSOLECLASS)] && $env(CONSOLECLASS) != "colour" } {
    set Display_Generic(color) "white"
    set Display_Generic(background) "white"

    set Display_Symbology(Actor) "black"
    set Display_Symbology(Agent) "black"
    set Display_Symbology(Role) "black"
    set Display_Symbology(Position) "black"
    set Display_Symbology(Resource) "black"
    set Display_Symbology(Task) "black"
    set Display_Symbology(Softgoal) "black"
    set Display_Symbology(Goal) "black"
    set Display_Symbology(Label) "black"
    set Display_Symbology(Alternative) "black"
    
    set Display_Node_Fill(Actor) "white"
    set Display_Node_Fill(Agent) "white"
    set Display_Node_Fill(Role) "white"
    set Display_Node_Fill(Position) "white"
    set Display_Node_Fill(Resource) "white"
    set Display_Node_Fill(Task) "white"
    set Display_Node_Fill(Softgoal) "white"
    set Display_Node_Fill(Goal) "white"
    set Display_Node_Fill(Label) "white"
    set Display_Node_Fill(Alternative) "white"
    
    set Display_Node_Enter(Actor) "black"
    set Display_Node_Enter(Agent) "black"
    set Display_Node_Enter(Role) "black"
    set Display_Node_Enter(Position) "black"
    set Display_Node_Enter(Resource) "black"
    set Display_Node_Enter(Task) "black"
    set Display_Node_Enter(Softgoal) "black"
    set Display_Node_Enter(Goal) "black"
    set Display_Node_Enter(Label) "black"
    set Display_Node_Enter(Alternative) "black"

} else {
    set Display_Generic(color) "wheat"
    set Display_Generic(background) "bisque"

    set Display_Symbology(Actor) "Blue"
    set Display_Symbology(Agent) "Blue"
    set Display_Symbology(Role) "Blue"
    set Display_Symbology(Position) "Blue"
    set Display_Symbology(Resource) "DarkSeaGreen"
    set Display_Symbology(Task) "CornflowerBlue"
    set Display_Symbology(Softgoal) "DarkOrange"
    set Display_Symbology(Goal) "Brown"
    set Display_Symbology(Label) "yellow"
    set Display_Symbology(Alternative) "Red"
    
    set Display_Node_Fill(Actor) "wheat"
    set Display_Node_Fill(Agent) "wheat"
    set Display_Node_Fill(Role) "wheat"
    set Display_Node_Fill(Position) "wheat"
    set Display_Node_Fill(Resource) "wheat"
    set Display_Node_Fill(Task) "wheat"
    set Display_Node_Fill(Softgoal) "wheat"
    set Display_Node_Fill(Goal) "wheat"
    set Display_Node_Fill(Label) "wheat"
    set Display_Node_Fill(Alternative) "wheat"

    set Display_Node_Enter(Actor) "lightGray"
    set Display_Node_Enter(Agent) "lightGray"
    set Display_Node_Enter(Role) "lightGray"
    set Display_Node_Enter(Position) "lightGray"
    set Display_Node_Enter(Resource) "lightGray"
    set Display_Node_Enter(Task) "lightGray"
    set Display_Node_Enter(Softgoal) "lightGray"
    set Display_Node_Enter(Goal) "lightGray"
    set Display_Node_Enter(Label) "lightGray"
    set Display_Node_Enter(Alternative) "lightGray"
}

######################################################################
# Node sizes
######################################################################

set Size_Type(large_size) 50
set Size_Type(default_size) 40
set Size_Type(small_size) 30
set Size_Type(tiny_size) 20

set Display_Size(Actor) $Size_Type(default_size)
set Display_Size(Agent) $Size_Type(default_size)
set Display_Size(Role) $Size_Type(default_size)
set Display_Size(Position) $Size_Type(default_size)
set Display_Size(Resource) $Size_Type(default_size)
set Display_Size(Task) $Size_Type(default_size)
set Display_Size(Goal) $Size_Type(default_size)
set Display_Size(Softgoal) $Size_Type(default_size)
set Display_Size(Label) $Size_Type(default_size)
set Display_Size(Alternative) $Size_Type(small_size)

set Display_Link_Size(Depend) 10
set Display_Link_Size(Task) 10
set Display_Link_Size(Means) 10
set Display_Link_Size(SGoal) 10

######################################################################
# Dependency colors
######################################################################

if {[info exists env(CONSOLECLASS)] && $env(CONSOLECLASS) != "colour" } {
    set Display_Depend_Color(Task) "black"
} else {
    set Display_Depend_Color(Task) "black"
}

######################################################################
# Context Containers
######################################################################

if {[info exists env(CONSOLECLASS)] && $env(CONSOLECLASS) != "colour" } {
    set Display_Context_Color(Generic) "black"
} else {
    set Display_Context_Color(Generic) "black"
}

set Display_Context_Bound(Generic) 20
set Display_Context_Bound(Actor) 20

######################################################################
# Counters for Unique ids
######################################################################

set Actor_No 0
set Agent_No 0
set Role_No 0
set Position_No 0
set Goal_No 0
set Resource_No 0
set Softgoal_No 0
set Task_No 0
set Label_No 0
set Alternative_No 0
## added by mike Number of objects
set Object_No 0
######################################################################
# Fonts
######################################################################

set Font_Type(large_font) "-*-times-bold-r-normal--*-180-*-*-*-*-*-*"
set Font_Type(default_font) "-*-times-bold-r-normal--*-140-*-*-*-*-*-*"
set Font_Type(small_font) "-*-times-bold-r-normal--*-100-*-*-*-*-*-*"
set Font_Type(tiny_font) "-*-times-bold-r-normal--*-80-*-*-*-*-*-*"
set Font_Type(text_font) "-Adobe-Times-medium-R-Normal-*-180-*"
set Font_Type(dialog_font) "-Adobe-Times-medium-R-Normal-*-180-*"

set Display_Font(Text) $Font_Type(text_font)
set Display_Font(Node,Label) $Font_Type(default_font)
set Display_Font(Node,Alternative) $Font_Type(default_font)
set Display_Font(Node,Actor) $Font_Type(default_font)
set Display_Font(Node,Agent) $Font_Type(default_font)
set Display_Font(Node,Role) $Font_Type(default_font)
set Display_Font(Node,Position) $Font_Type(default_font)
set Display_Font(Node,Resource) $Font_Type(default_font)
set Display_Font(Node,Task) $Font_Type(default_font)
set Display_Font(Node,Goal) $Font_Type(default_font)
set Display_Font(Node,Softgoal) $Font_Type(default_font)
set Display_Font(Link) $Font_Type(default_font)
set Display_Font(Dialog) $Font_Type(dialog_font)

######################################################################
# Global options
######################################################################

option add *buttons*relief raised userDefault
option add *buttons*borderWidth 2 userDefault
option add *Frame.borderWidth 2 userDefault
option add *Text*background bisque userDefault
option add *Text*wrap word userDefault
option add *Text.relief sunken userDefault
option add *Text.borderWidth 2 userDefault
option add *Scrollbar*relief sunken userDefault

option add *Listbox*font \
    "-*-helvetica-medium-r-normal-*-14-*-*-*-p-*-iso8859-1" userDefault
option add *Entry*font \
    "-*-helvetica-bold-r-normal-*-14-*-*-*-p-*-iso8859-1" userDefault
option add *Label*font \
    "-*-helvetica-bold-r-normal-*-14-*-*-*-p-*-iso8859-1" userDefault
option add *Button*font \
    "-*-helvetica-bold-r-normal-*-14-*-*-*-p-*-iso8859-1" userDefault
option add *Menu*font \
    "-*-helvetica-bold-r-normal-*-14-*-*-*-p-*-iso8859-1" userDefault
option add *MenuButton*font \
    "-*-helvetica-bold-r-normal-*-14-*-*-*-p-*-iso8859-1" userDefault
    
###############################################################    
# Default Display Position
###############################################################
set Default_Pos "50 50"

###############################################################    
# All Object Types
###############################################################

set Object_List "Actor Agent Role Position Label Alternative Resource Task Goal Softgoal"
set Valid_Context_List "Actor Agent Role Position"
