
# (C) Copyright 1996 University of Toronto
# Permission to use this software for non-commerical purposes is hereby
# granted.  This software is provided "as is" without express or implied
# warranty.  This software may not be transferred or sublicensed.
# Any enhancements made by the recipient of this software shall be
# shared with the original copyright holders.

# Acknowledgements
# This software was developed as part of the project entitled "Models
# for Supporting the Analysis and Redesign of Business Processes" funded
# by the Information Technology Research Centre of Ontario.
# Principal investigators for the project are John Mylopoulos, Ray
# Reiter, Hector Levesque, Eric Yu, and Yves Lesperance.
# The i* modelling framework (including the Strategic Dependency and
# Strategic Rationale models) was developed by Eric Yu in his doctoral
# dissertation, Department of Computer Science, University of Toronto,
# 1995.
# This software was originally written by Chen Wang as part of his
# Master degree requirement.
# Substantial modifications and enhancements were made by Godfrey Cheng
# and Seyil Yoon, who worked as summer students on this project.
# The "repository" part of the code was originally written by Brian Kramer and
# subsequently modified by Chen Wang, Seyil Yoon, and Godfrey Cheng.


proc move_node {canvas tag dx dy} {
    $canvas move main$tag $dx $dy
    $canvas move misc$tag $dx $dy
    $canvas move label$tag $dx $dy
}

proc move_from_node_link {canvas curtag nx ny} {
    global Display_Node_Pos 

    do_move_from_link $canvas $curtag Depend_From $nx $ny
    do_move_from_link $canvas $curtag Task_From $nx $ny
    do_move_from_link $canvas $curtag Means_From $nx $ny
    do_move_from_link $canvas $curtag SGoal_From $nx $ny
    do_move_from_link $canvas $curtag Part_From $nx $ny
    do_move_from_link $canvas $curtag Play_From $nx $ny
    do_move_from_link $canvas $curtag Cover_From $nx $ny
    do_move_from_link $canvas $curtag Isa_From $nx $ny
    do_move_from_link $canvas $curtag Occupy_From $nx $ny
}

proc move_to_node_link {canvas curtag nx ny} {
    global Display_Node_Pos

    do_move_to_link $canvas $curtag Depend_To $nx $ny
    do_move_to_link $canvas $curtag Task_To $nx $ny
    do_move_to_link $canvas $curtag Means_To $nx $ny
    do_move_to_link $canvas $curtag SGoal_To $nx $ny
    do_move_to_link $canvas $curtag Part_To $nx $ny
    do_move_to_link $canvas $curtag Play_To $nx $ny
    do_move_to_link $canvas $curtag Cover_To $nx $ny
    do_move_to_link $canvas $curtag Isa_To $nx $ny
    do_move_to_link $canvas $curtag Occupy_To $nx $ny
}

proc node_start_drag {canvas x y} {
    global _last_x _last_y _node_drag_tag
    
    set _node_drag_tag [lindex [$canvas gettags current] 0]
    set _last_x $x
    set _last_y $y
}

proc node_drag {canvas x y} {
    global _last_x _last_y _node_drag_tag Display_Node_Pos
    global Display_Node_Context Display_Node_Text

    set cur_x [expr int($x)]
    set cur_y [expr int($y)]
    set curtag $_node_drag_tag

    set dx [expr $cur_x-$_last_x]
    set dy [expr $cur_y-$_last_y]

    move_node $canvas $curtag $dx $dy

    set nx [expr int([lindex $Display_Node_Pos($canvas,$curtag) 0])]
    set ny [expr int([lindex $Display_Node_Pos($canvas,$curtag) 1])]

#    set tempdx [expr int($dx)]
#    puts $tempdx
#    set tempdy [expr int($dy)]
#    puts $tempdy
#    incr nx $tempdx
#    incr ny $tempdy
    incr nx [expr int($dx)]
    incr ny [expr int($dy)]

    move_from_node_link $canvas $curtag $nx $ny
    move_to_node_link $canvas $curtag $nx $ny

    adjust_context_region $canvas $curtag

    set Display_Node_Pos($canvas,$curtag) "$nx $ny"
        
    set _last_x $x
    set _last_y $y

    if {[info exists Display_Node_Context($canvas,$curtag)]} {
       adjust_context_region $canvas $Display_Node_Context($canvas,$curtag)
    }

    check_adjust_canvas $canvas $nx $ny
}

############################################################################
# Edit Node Names
############################################################################

proc rename_node {canvas type tag old_node_name} {
    global User_Input New_Node_Name Display_Node_Text Exist_Object
    global Display_Node_ID Display_Node_Text
    
    set New_Node_Name $User_Input(label)
    set txt [lindex $Display_Node_Text($canvas,$tag) 0]    
    
    ## if validate_label .....
    ##### kb name old_node_name New_Node_Name
    
    # set tmp [lsearch $Exist_Object($canvas,$type) $old_node_name]    
    # set Exist_Object($canvas,$type)
    #    [lreplace $Exist_Object($canvas,$type) $tmp $tmp $New_Node_Name]
    
    # set tmp2 [lindex $Display_Node_Text($canvas,$tag) 1]

    # set Display_Node_Text($canvas,$tag) "$txt $New_Node_Name"

    # set tmp $Display_Node_ID($canvas,$tmp2)
    # unset Display_Node_ID($canvas,$tmp2)
    # set Display_Node_ID($canvas,$New_Node_Name) $tmp
    
     $canvas itemconfigure $txt -text $New_Node_Name
}

proc edit_current_node {canvas tag type} {
    global Input_No User_Input Display_Font Frame_no New_Node_Name
    global Display_Node_Text
    
    incr Input_No
    incr Frame_no
    
    ##### create new dialog window #####
    
    set name [lindex $Display_Node_Text($canvas,$tag) 1]
    set text [toplevel .text$Input_No]
    wm title $text "Edit Node"
    # wm minsize $text 416 167
    
    pack [label $text.lbl -text ""]
    pack [canvas $text.c -height 30m] -padx 3m

    ##### create Cancel button #####
    
    button $text.cancel -text Cancel -command "set New_Node_Name null"
    pack $text.cancel -pady 2m -ipadx 5m -ipady 2m

    ##### create tabbed entry #####
    
    set New_Node_Name ""
    set fn $Display_Font(Text)
    set f [frame $text.c.frame$Frame_no]
    
    
    set topf [create_tabbed_frame $f 0]
    set entry [entry $topf.entry -textvariable User_Input(label) \
        -relief sunken -width 20 -font $fn]
    
    pack [label $topf.label -text "Please enter a new name for $type $name:\n" \
        -font $fn] 
    pack $entry
    pack $topf -side top -fill x
    
    bind $entry <Return> "rename_node $canvas $type $tag $name"    
    
    insert_text_frame $text $f
    
    ##### grab window #####
    
    set oldFocus [focus]
    catch {
        grab $text
    }
    focus $text
    
    tkwait variable New_Node_Name
    destroy $text
    set User_Input(label) ""
    focus $oldFocus
}

proc menu_edit_current_node {canvas menu tag type} {
    unpost_node_menu $canvas $menu

    edit_current_node $canvas $tag $type
}

############################################################################
# Deleting Nodes
############################################################################

proc delete_all_node_links {canvas tag l} {
   global Display_Node_Link Display_Links

   if {[info exists Display_Node_Link($canvas,$tag,$l)]} {
       foreach link $Display_Node_Link($canvas,$tag,$l) {
           delete_display_link $canvas $link
       }
   }
}

proc delete_current_node {canvas tag type} {
   global Display_Nodes Display_Node_Pos Display_Node_ID Display_Node_Main Display_Node_Line
   global Display_Node_Text Display_Node_Tag Display_Node_Link Display_Node_Context Exist_Object Display_Font_Tag

   $canvas delete main$tag
   $canvas delete misc$tag
   $canvas delete label$tag

   # Clean up the links 

   delete_all_node_links $canvas $tag Depend_From
   delete_all_node_links $canvas $tag Task_From
   delete_all_node_links $canvas $tag Means_From
   delete_all_node_links $canvas $tag SGoal_From
   delete_all_node_links $canvas $tag Part_From
   delete_all_node_links $canvas $tag Play_From
   delete_all_node_links $canvas $tag Cover_From
   delete_all_node_links $canvas $tag Isa_From
   delete_all_node_links $canvas $tag Occupy_From

   delete_all_node_links $canvas $tag Depend_To
   delete_all_node_links $canvas $tag Task_To
   delete_all_node_links $canvas $tag Means_To
   delete_all_node_links $canvas $tag SGoal_To
   delete_all_node_links $canvas $tag Part_To
   delete_all_node_links $canvas $tag Play_To
   delete_all_node_links $canvas $tag Cover_To
   delete_all_node_links $canvas $tag Isa_To
   delete_all_node_links $canvas $tag Occupy_To
# WINGS BEGIN
   unset Display_Node_Pos($canvas,$tag)
   set s $Display_Node_Main($canvas,$tag)
   unset Display_Node_Main($canvas,$tag)
    set Display_Font_Tag($tag) 1
   unset Display_Font_Tag($tag)
   set t [lsearch $Display_Nodes($canvas,$type) $s]

   set Display_Nodes($canvas,$type) \
        [lreplace $Display_Nodes($canvas,$type) [expr $t] [expr $t]]

   set text [lindex $Display_Node_Text($canvas,$tag) 1]
   unset Display_Node_Text($canvas,$tag)

   unset Display_Node_ID($canvas,$text)

   if {[info exists Display_Node_Line($canvas,$tag)]} {
        unset Display_Node_Line($canvas,$tag)
   }

   unset Display_Node_Tag($canvas,$s)

   foreach x [array names Display_Node_Link] {
        if [string match $canvas,$tag* $x] {
                unset Display_Node_Link($x)
        }
    }

    unset Display_Node_Context($canvas,$tag)

    foreach x [array names Display_Node_Context] {
        if {$Display_Node_Context($x) == $tag} {
                set Display_Node_Context($x) ""
        }
    }

    set t [lsearch $Exist_Object($canvas,$type) $text]
    set Exist_Object($canvas,$type) [lreplace $Exist_Object($canvas,$type) [expr $t] [expr $t]]
kb destroy $text
# WINGS END
}

proc menu_delete_current_node {canvas menu tag type} {
    global dlgButton Display_Node_Text 

    unpost_node_menu $canvas $menu

    set txt [lindex $Display_Node_Text($canvas,$tag) 1]
    
    dialogBox .dlg "Delete Node" "Delete $type $txt?\n" 1 Yes No    
    
    if {$dlgButton == 0} {    
        delete_current_node $canvas $tag $type
    }    
}

############################################################################
# Node context subroutines
############################################################################

proc menu_create_context_region {canvas menu tag} {
    unpost_node_menu $canvas $menu

    create_context_region $canvas $tag
    lower_canvas_background $canvas
}

proc menu_destroy_context_region {canvas menu tag} {
    unpost_node_menu $canvas $menu

    destroy_context_region $canvas $tag
    lower_canvas_background $canvas
}

############################################################################
# Binding Nodes
############################################################################


proc change_Font_Tag {tag obj font canvas} {
   global Display_Font_Tag Display_Node_Text Font_Type
   global Display_Size
    
   set Display_Font_Tag($tag) $Font_Type(${font}_font)
            
   set txt [lindex $Display_Node_Text($canvas,$tag) 0]
   puts ${font}_font              
   $canvas itemconfigure $txt -font $Font_Type(${font}_font) \
                    -width [expr $Display_Size($obj)*1.75]

}


proc post_node_menu {canvas m x y} {
    catch {
        focus $m
        $m post [expr $x + [winfo rootx $canvas]] \
               [expr $y + [winfo rooty $canvas]]
        grab $canvas
    } msg
}

proc unpost_node_menu {canvas m} {
    catch {
        $m unpost
        grab release $canvas
        focus $canvas
    }
}

proc create_node_menu {canvas tag type} {
   set menu [menu $canvas.${tag}_menu -tearoff n]

   $menu add command -label "Dismiss" -command \
           "unpost_node_menu $canvas $menu"
    
   $menu add separator

   $menu add command -label "Delete Node" -command \
        "menu_delete_current_node $canvas $menu $tag $type"
        
   $menu add command -label "Edit Node" -command \
        "menu_edit_current_node $canvas $menu $tag $type"
   
   $menu add separator

   $menu add command -label "Create Context" -command \
        "menu_create_context_region $canvas $menu $tag"
        
   $menu add command -label "Delete Context" -command \
        "menu_destroy_context_region $canvas $menu $tag"

   $menu add separator

   $menu add cascade -label "Change Font" -menu \
       $menu.subFont

    menu $menu.subFont -tearoff n
    $menu.subFont add command -label "Large"  -command \
       "change_Font_Tag $tag $type large $canvas"
    $menu.subFont add command -label "Default" -command \
       "change_Font_Tag $tag $type default $canvas" 
    $menu.subFont add command -label "Small" -command \
       "change_Font_Tag $tag $type small $canvas" 
    $menu.subFont add command -label "Tiny" -command \
       "change_Font_Tag $tag $type tiny $canvas"
#"changeFont $obj tiny $canvas"           

 
   return $menu
}

proc bind_node {canvas tag type} {
   global Display_Node_Fill Display_Node_Enter
   
   set ent_fill $Display_Node_Enter($type)
   set normal_fill $Display_Node_Fill($type)

   set menu [create_node_menu $canvas $tag $type]

   $canvas bind main$tag <Any-Enter> "
       $canvas itemconfigure current -fill $ent_fill
   "

   $canvas bind main$tag <Any-Leave> "
       $canvas itemconfigure current -fill $normal_fill
   "
    
   $canvas bind main$tag <Button-1> "
       node_start_drag $canvas %x %y
   "
 
   $canvas bind main$tag <B1-Motion> "
       node_drag $canvas %x %y
   "

   $canvas bind main$tag <ButtonPress-3> "
       post_node_menu $canvas $menu %x %y
   "

   $canvas bind main$tag <ButtonRelease-3> "
       unpost_node_menu $canvas $menu
   "
}

############################################################################
# Generic Nodes
############################################################################  

proc get_node_tags {tag} {
   return [list $tag main$tag node item]
}

proc get_misc_tags {tag} {
   return [list $tag misc$tag node item]
}

proc get_label_tags {tag} {
   return [list $tag label$tag label item]
}

proc is_valid_context {canvas tag} {
    global Display_Nodes Display_Node_Main

    if {[info exists Display_Nodes($canvas,Label)]} {
        if {[lsearch $Display_Nodes($canvas,Label) \
                $Display_Node_Main($canvas,$tag)] != -1} {
            return 1
        } else {
            if {[info exists Display_Nodes($canvas,Actor)]} {
                if {[lsearch $Display_Nodes($canvas,Actor) \
                        $Display_Node_Main($canvas,$tag)] != -1} {
                    return 1
                } else {
                    return 0
                }
            } else {
                return 0
            }
        }
    } else {
        if {[info exists Display_Nodes($canvas,Actor)]} {
            if {[lsearch $Display_Nodes($canvas,Actor) \
                    $Display_Node_Main($canvas,$tag)] != -1} {
                return 1
            } else {
                return 0
            }
        } else {
            return 0
        }
    }
}

proc extra_tag {canvas tag} {
    global Display_Nodes Display_Node_Context

    if {$tag != ""} {
        if {[is_valid_context $canvas $tag] == "1"} {
            if {[info exists Display_Node_Context($canvas,$tag)]} {
                return "$tag [extra_tag $canvas  \
                    $Display_Node_Context($canvas,$tag)]"
            } else {
                return "$tag"
            }
        } else {
            return "$tag"
        }
    } else {
        return ""
    }
}

############################################################################
# Label Nodes
############################################################################  

proc get_label_node_tags {tag} {
   return "[get_node_tags $tag] Label"
}

proc get_label_label_tags {tag} {
   return "[get_label_tags $tag] Label"
}

proc get_label_misc_tags {tag} {
   return "[get_misc_tags $tag] Label"
}

proc create_label_node {canvas x y tag {label ""} {drag 1} {etag ""}} {
   global Display_Symbology Display_Size Display_Node_Fill 
   global Display_Node_Pos Display_Nodes Display_Node_Tag
   global Display_Node_Text Display_Node_ID Display_Node_Context
   global Display_Font Font_Type Display_Node_Line
   global Display_Node_Main Display_Generic Object_No

   set fn $Display_Font(Node,Label)

   if {$label == ""} {
       set text $tag
   } else {
       set text $label
   }

    if {$Object_No < 5} {
	set number $Object_No
    } else {
	set number [expr fmod($Object_No,5)]
    }
    set x [expr $x + $number*10]
    set y [expr $y +$number*10]

   set color $Display_Generic(background)
   set width [expr $Display_Size(Label)*3]
   set height $Display_Size(Label)
   set fill $Display_Node_Fill(Label)
   set extra [extra_tag $canvas $etag]
  
   set main [$canvas create rectangle [expr $x-[expr $width*0.5]] \
       [expr $y-[expr $height*0.5]] [expr $x+[expr $width*0.5]] \
       [expr $y+[expr $height*0.5]] -fill $fill \
       -tag "[get_label_node_tags $tag] $extra"]

   set line [$canvas create line [expr $x-[expr $width*0.5]] \
       [expr $y-[expr $height*0.5]] [expr $x+[expr $width*0.5]] \
       [expr $y-[expr $height*0.5]] [expr $x+[expr $width*0.5]] \
       [expr $y+[expr $height*0.5]] [expr $x-[expr $width*0.5]] \
       [expr $y+[expr $height*0.5]] [expr $x-[expr $width*0.5]] \
       [expr $y-[expr $height*0.5]] -fill $color \
       -tag "[get_label_misc_tags $tag] $extra"]

### changed by mike: _ is changed to line break
    set listext [split $text "|"]
    set text3 [join $listext "\n"]
    set t \
     [$canvas create text \
      $x $y -text "$text3"  -width [expr $width*1.9] -font $fn\
      -tag "[get_label_label_tags $tag] $extra"]

   lappend Display_Nodes($canvas,Label) $main
   set Display_Node_Tag($canvas,$main) $tag
   set Display_Node_Main($canvas,$tag) $main
   set Display_Node_Line($canvas,$tag) $line

   set Display_Node_Pos($canvas,$tag) "$x $y"

   set Display_Node_Text($canvas,$tag) "$t $text"
   set Display_Node_ID($canvas,$text) $tag

   set Display_Node_Context($canvas,$tag) "$etag"
   if {$drag == 1} {
       bind_node $canvas $tag Label
   }

   if {$extra != ""} {
       foreach i $extra {
         adjust_context_region $canvas $i
       }
   }

   check_adjust_canvas $canvas $x $y
}

############################################################################
# Alternative Nodes
############################################################################  

proc get_alternative_node_tags {tag} {
   return "[get_node_tags $tag] Alternative"
}

proc get_alternative_label_tags {tag} {
   return "[get_label_tags $tag] Alternative"
}

proc get_alternative_misc_tags {tag} {
   return "[get_misc_tags $tag] Alternative"
}

proc create_alternative_node {canvas x y tag {label ""} {drag 1} {etag ""}} {
   global Display_Symbology Display_Size Display_Node_Fill 
   global Display_Node_Pos Display_Nodes Display_Node_Tag
   global Display_Node_Text Display_Node_ID Display_Node_Context
   global Display_Font Font_Type Display_Node_Line
   global Display_Node_Main Object_No
    global Display_Font_Tag

###added by mike
    if {[info exists Display_Font_Tag($tag)]} {
	set fn $Display_Font_Tag($tag)
    } else {
	set fn $Display_Font(Node,Alternative)
    }
# puts $tag
# puts $fn

   if {$label == ""} {
       set text $tag
   } else {
       set text $label
   }

    if {$Object_No < 5} {
	set number $Object_No
    } else {
	set number [expr fmod($Object_No,5)]
    }
    set x [expr $x + $number*10]
    set y [expr $y +$number*10]

   set color $Display_Symbology(Alternative)  
   set width $Display_Size(Alternative)
   set height $Display_Size(Alternative)
   set fill $Display_Node_Fill(Alternative)
   set extra [extra_tag $canvas $etag]
   
   set main [$canvas create polygon [expr $x-$width] $y \
    $x [expr $y-$height] \
    [expr $x+$width] $y \
    $x [expr $y+$height] \
    [expr $x-$width] $y \
          -fill $fill -tag \
          "[get_alternative_node_tags $tag] $extra"]

   set line [$canvas create line [expr $x-$width] $y \
    $x [expr $y-$height] \
    [expr $x+$width] $y \
    $x [expr $y+$height] \
    [expr $x-$width] $y \
          -fill $color -width 2 -tag \
          "[get_alternative_misc_tags $tag] $extra"]

### changed by mike: _ is changed to line break
    set listext [split $text "|"]
    set text3 [join $listext "\n"]
    set t \
     [$canvas create text \
      $x $y -text "$text3"  -width [expr $width*0.9] -font $fn\
      -tag "[get_alternative_label_tags $tag] $extra"]

   lappend Display_Nodes($canvas,Alternative) $main
   set Display_Node_Tag($canvas,$main) $tag
   set Display_Node_Main($canvas,$tag) $main
   set Display_Node_Line($canvas,$tag) $line

   set Display_Node_Pos($canvas,$tag) "$x $y"

   set Display_Node_Text($canvas,$tag) "$t $text"
   set Display_Node_ID($canvas,$text) $tag

   set Display_Node_Context($canvas,$tag) "$etag"

   if {$drag == 1} {
       bind_node $canvas $tag Alternative
   }

   if {$extra != ""} {
       foreach i $extra {
         adjust_context_region $canvas $i
       }
   }
   check_adjust_canvas $canvas $x $y
}

############################################################################
# Actor Nodes
############################################################################  

proc get_actor_node_tags {tag} {
   return "[get_node_tags $tag] Actor"
}

proc get_actor_label_tags {tag} {
   return "[get_label_tags $tag] Actor"
}

proc create_actor_node {canvas x y tag {label ""} {drag 1} {etag ""}} {
   global Display_Symbology Display_Size Display_Node_Fill 
   global Display_Node_Pos Display_Nodes Display_Node_Tag
   global Display_Node_Text Display_Node_ID Display_Node_Context
   global Display_Font Font_Type
   global Display_Node_Main Object_No
    global Display_Font_Tag

###added by mike
    if {[info exists Display_Font_Tag($tag)]} {
	set fn $Display_Font_Tag($tag)
    } else {
	set fn $Display_Font(Node,Actor)
    }
# puts $tag
# puts $fn

   if {$label == ""} {
       set text $tag
   } else {
       set text $label
   }

    if {$Object_No < 5} {
	set number $Object_No
    } else {
	set number [expr fmod($Object_No,5)]
    }
    set x [expr $x + $number*10]
    set y [expr $y +$number*10]

   set color $Display_Symbology(Actor)  
   set rad $Display_Size(Actor)
   set fill $Display_Node_Fill(Actor)
   set extra [extra_tag $canvas $etag]

   set main [$canvas create oval \
    [expr $x+$rad] [expr $y+$rad] \
    [expr $x-$rad] [expr $y-$rad] -width 2 \
            -outline $color -fill $fill -tag \
            "[get_actor_node_tags $tag] $extra"]
### changed by mike: _ is changed to line break
    set listext [split $text "|"]
    puts $listext
    set text3 [join $listext "\n"]
    set t \
     [$canvas create text \
      $x $y -text "$text3"  -width [expr $rad*1.75] -font $fn\
      -tag "[get_actor_label_tags $tag] $extra"]


   lappend Display_Nodes($canvas,Actor) $main
   set Display_Node_Tag($canvas,$main) $tag
   set Display_Node_Main($canvas,$tag) $main

   set Display_Node_Pos($canvas,$tag) "$x $y"

   set Display_Node_Text($canvas,$tag) "$t $text"
   set Display_Node_ID($canvas,$text) $tag

   set Display_Node_Context($canvas,$tag) "$etag"

   if {$drag == 1} {
       bind_node $canvas $tag Actor
   }

   if {$extra != ""} {
       foreach i $extra {
         adjust_context_region $canvas $i
       }
   }

   check_adjust_canvas $canvas $x $y
}
############################################################################
# Agent Nodes
############################################################################  

proc get_agent_node_tags {tag} {
   return "[get_node_tags $tag] Agent"
}

proc get_agent_label_tags {tag} {
   return "[get_label_tags $tag] Agent"
}

proc get_agent_misc_tags {tag} {
   return "[get_misc_tags $tag] Agent"
}

proc create_agent_node {canvas x y tag {label ""} {drag 1} {etag ""}} {
   global Display_Symbology Display_Size Display_Node_Fill 
   global Display_Node_Pos Display_Nodes Display_Node_Tag
   global Display_Node_Text Display_Node_ID Display_Node_Context
   global Display_Font Font_Type Display_Node_Line PI
   global Display_Node_Main Object_No 
    global Display_Font_Tag


###added by mike
    if {[info exists Display_Font_Tag($tag)]} {
	set fn $Display_Font_Tag($tag)
    } else {
	set fn $Display_Font(Node,Agent)
    }
# puts $tag
# puts $fn

   if {$label == ""} {
       set text $tag
   } else {
       set text $label
   }

    if {$Object_No < 5} {
	set number $Object_No
    } else {
	set number [expr fmod($Object_No,5)]
    }
    set x [expr $x + $number*10]
    set y [expr $y +$number*10]

   set color $Display_Symbology(Agent)  
   set rad $Display_Size(Agent)
   set tilt sin([expr $PI/4])*$rad
   set fill $Display_Node_Fill(Agent)
   set extra [extra_tag $canvas $etag]
 
   set main [$canvas create oval \
    [expr $x+$rad] [expr $y+$rad] \
    [expr $x-$rad] [expr $y-$rad] -width 2 \
            -outline $color -fill $fill -tag \
      "[get_agent_node_tags $tag] $extra"]

   set line [$canvas create line [expr $x-$tilt] [expr $y-$tilt] \
      [expr $x+$tilt] [expr $y-$tilt] -fill $color -width 2 \
      -tag "[get_agent_misc_tags $tag] $extra"]

### changed by mike: _ is changed to line break
    set listext [split $text "|"]
    set text3 [join $listext "\n"]
    set t \
     [$canvas create text \
      $x $y -text "$text3"  -width [expr $rad*1.75] -font $fn\
      -tag "[get_actor_label_tags $tag] $extra"]

   lappend Display_Nodes($canvas,Agent) $main
   set Display_Node_Tag($canvas,$main) $tag
   set Display_Node_Main($canvas,$tag) $main
   set Display_Node_Line($canvas,$tag) $line

   set Display_Node_Pos($canvas,$tag) "$x $y"

   set Display_Node_Text($canvas,$tag) "$t $text"
   set Display_Node_ID($canvas,$text) $tag

   set Display_Node_Context($canvas,$tag) "$etag"

   if {$drag == 1} {
       bind_node $canvas $tag Agent
   }

   if {$extra != ""} {
       foreach i $extra {
         adjust_context_region $canvas $i
       }
   }

   check_adjust_canvas $canvas $x $y
}

############################################################################
# Role Nodes
############################################################################  

proc get_role_node_tags {tag} {
   return "[get_node_tags $tag] Role"
}

proc get_role_label_tags {tag} {
   return "[get_label_tags $tag] Role"
}

proc get_role_misc_tags {tag} {
   return "[get_misc_tags $tag] Role"
}

proc create_role_node {canvas x y tag {label ""} {drag 1} {etag ""}} {
   global Display_Symbology Display_Size Display_Node_Fill 
   global Display_Node_Pos Display_Nodes Display_Node_Tag
   global Display_Node_Text Display_Node_ID Display_Node_Context
   global Display_Font Font_Type Display_Node_Line PI
   global Display_Node_Main Object_No
    global Display_Font_Tag

###added by mike
    if {[info exists Display_Font_Tag($tag)]} {
	set fn $Display_Font_Tag($tag)
    } else {
	set fn $Display_Font(Node,Role)
    }
# puts $tag
# puts $fn

   if {$label == ""} {
       set text $tag
   } else {
       set text $label
   }

    if {$Object_No < 5} {
	set number $Object_No
    } else {
	set number [expr fmod($Object_No,5)]
    }
    set x [expr $x + $number*10]
    set y [expr $y +$number*10]

   set color $Display_Symbology(Role)  
   set rad $Display_Size(Role)
   set tilt sin([expr $PI/4])*$rad
   set fill $Display_Node_Fill(Role)
   set extra [extra_tag $canvas $etag]
 
   set main [$canvas create oval \
    [expr $x+$rad] [expr $y+$rad] \
    [expr $x-$rad] [expr $y-$rad] -width 2 -outline $color \
            -fill $fill -tag \
            "[get_role_node_tags $tag] $extra"]

   set line [$canvas create line [expr $x-$tilt] [expr $y+$tilt] \
      [expr $x+$tilt] [expr $y+$tilt] -fill $color -width 2 \
      -tag "[get_role_misc_tags $tag] $extra"]

### changed by mike: _ is changed to line break
    set listext [split $text "|"]
    set text3 [join $listext "\n"]
    set t \
     [$canvas create text \
      $x $y -text "$text3"  -width [expr $rad*1.75] -font $fn\
      -tag "[get_agent_label_tags $tag] $extra"]

   lappend Display_Nodes($canvas,Role) $main
   set Display_Node_Tag($canvas,$main) $tag
   set Display_Node_Main($canvas,$tag) $main
   set Display_Node_Line($canvas,$tag) $line

   set Display_Node_Pos($canvas,$tag) "$x $y"

   set Display_Node_Text($canvas,$tag) "$t $text"
   set Display_Node_ID($canvas,$text) $tag

   set Display_Node_Context($canvas,$tag) "$etag"

   if {$drag == 1} {
       bind_node $canvas $tag Role
   }

   if {$extra != ""} {
       foreach i $extra {
         adjust_context_region $canvas $i
       }
   }

   check_adjust_canvas $canvas $x $y
}

############################################################################
# Position Nodes
############################################################################  

proc get_position_misc_tags {tag} {
   return "[get_misc_tags $tag] Position"
}

proc get_position_node_tags {tag} {
   return "[get_node_tags $tag] Position"
}

proc get_position_label_tags {tag} {
   return "[get_label_tags $tag] Position"
}

proc create_position_node {canvas x y tag {label ""} {drag 1} {etag ""}} {
   global Display_Symbology Display_Size Display_Node_Fill 
   global Display_Node_Pos Display_Nodes Display_Node_Tag
   global Display_Node_Text Display_Node_ID Display_Node_Context
   global Display_Font Font_Type Display_Node_Line PI
   global Display_Node_Main Display_Node_Line2 Object_No
    global Display_Font_Tag
### Display_Node_Line2 added by mike

###added by mike
    if {[info exists Display_Font_Tag($tag)]} {
	set fn $Display_Font_Tag($tag)
    } else {
	set fn $Display_Font(Node,Position)
    }
# puts $tag
# puts $fn


   if {$label == ""} {
       set text $tag
   } else {
       set text $label
   }

    if {$Object_No < 5} {
	set number $Object_No
    } else {
	set number [expr fmod($Object_No,5)]
    }
    set x [expr $x + $number*10]
    set y [expr $y +$number*10]

   set color $Display_Symbology(Position)  
   set rad $Display_Size(Position)
   set tilt sin([expr $PI/4])*$rad
   set fill $Display_Node_Fill(Position)
   set extra [extra_tag $canvas $etag]
### the new one: circle with two lines by mike
   set main [$canvas create oval \
    [expr $x+$rad] [expr $y+$rad] \
    [expr $x-$rad] [expr $y-$rad] -width 2 -outline $color \
            -fill $fill -tag \
            "[get_position_node_tags $tag] $extra"]

  set line [$canvas create line [expr $x-$tilt] [expr $y-$tilt] \
      [expr $x+$tilt] [expr $y-$tilt] -fill $color -width 2 \
      -tag "[get_position_misc_tags $tag] $extra"]

  set line2 [$canvas create line [expr $x-$tilt] [expr $y+$tilt] \
      [expr $x+$tilt] [expr $y+$tilt] -fill $color -width 2 \
		-tag "[get_position_misc_tags $tag] $extra"]

  

#### the old one 
#   set color $Display_Symbology(Position)  
#   set rad $Display_Size(Position)
#   set tilt [expr $rad/2]
#   set tilts [expr $tilt*2]
#   set fill $Display_Node_Fill(Position)
#   set extra [extra_tag $canvas $etag]
# 
#   set main [$canvas create polygon \
        [expr $x-$tilt] [expr $y-$tilts] [expr $x+$tilt] [expr $y-$tilts] \
        [expr $x+$tilt] [expr $y-$tilt] [expr $x+$tilts] [expr $y-$tilt] \
        [expr $x+$tilts] [expr $y+$tilt] [expr $x+$tilt] [expr $y+$tilt] \
        [expr $x+$tilt] [expr $y+$tilts] [expr $x-$tilt] [expr $y+$tilts] \
        [expr $x-$tilt] [expr $y+$tilt] [expr $x-$tilts] [expr $y+$tilt] \
        [expr $x-$tilts] [expr $y-$tilt] [expr $x-$tilt] [expr $y-$tilt] \
        [expr $x-$tilt] [expr $y-$tilts] \
        -width 2 -outline $color -fill $fill -smooth 1 \
        -tag "[get_position_node_tags $tag] $extra"]
            
#   set line [$canvas create line \
        [expr $x-$tilt] [expr $y-$tilts] [expr $x+$tilt] [expr $y-$tilts] \
        [expr $x+$tilt] [expr $y-$tilt] [expr $x+$tilts] [expr $y-$tilt] \
        [expr $x+$tilts] [expr $y+$tilt] [expr $x+$tilt] [expr $y+$tilt] \
        [expr $x+$tilt] [expr $y+$tilts] [expr $x-$tilt] [expr $y+$tilts] \
        [expr $x-$tilt] [expr $y+$tilt] [expr $x-$tilts] [expr $y+$tilt] \
        [expr $x-$tilts] [expr $y-$tilt] [expr $x-$tilt] [expr $y-$tilt] \
        [expr $x-$tilt] [expr $y-$tilts] \
        -fill $color -width 2 -smooth 1 \
        -tag "[get_position_misc_tags $tag] $extra"]

### changed by mike: _ is changed to line break
set listext [split $text "|"]
    set text3 [join $listext "\n"]
    set t \
     [$canvas create text \
      $x $y -text "$text3" -width [expr $rad*1.75] -font $fn\
      -tag "[get_position_label_tags $tag] $extra"]
 
   lappend Display_Nodes($canvas,Position) $main
   set Display_Node_Tag($canvas,$main) $tag
   set Display_Node_Main($canvas,$tag) $main
   set Display_Node_Line($canvas,$tag) $line
### the next line was added by mike
   set Display_Node_Line2($canvas,$tag) $line2

   set Display_Node_Pos($canvas,$tag) "$x $y"

   set Display_Node_Text($canvas,$tag) "$t $text"
   set Display_Node_ID($canvas,$text) $tag

   set Display_Node_Context($canvas,$tag) "$etag"

   if {$drag == 1} {
       bind_node $canvas $tag Position
   }

   if {$extra != ""} {
       foreach i $extra {
         adjust_context_region $canvas $i
       }
   }

   check_adjust_canvas $canvas $x $y
}

############################################################################
# Resource Nodes
############################################################################  

proc get_resource_node_tags {tag} {
   return "[get_node_tags $tag] Resource"
}

proc get_resource_label_tags {tag} {
   return "[get_label_tags $tag] Resource"
}
       
proc create_resource_node {canvas x y tag {label ""} {drag 1} {etag ""}} {
   global Display_Symbology Display_Size Display_Node_Fill 
   global Display_Node_Pos Display_Nodes Display_Node_Tag
   global Display_Node_Text Display_Node_ID Display_Node_Context
   global Display_Font Font_Type Display_Font_Tag
   global Display_Node_Main Object_No

###added by mike
    if {[info exists Display_Font_Tag($tag)]} {
	set fn $Display_Font_Tag($tag)
    } else {
	set fn $Display_Font(Node,Resource)
    }
# puts $tag
# puts $fn


   if {$label == ""} {
       set text $tag
   } else {
       set text $label
   }

    if {$Object_No < 5} {
	set number $Object_No
    } else {
	set number [expr fmod($Object_No,5)]
    }
    set x [expr $x + $number*10]
    set y [expr $y +$number*10]

   set color $Display_Symbology(Resource)  
   set width $Display_Size(Resource)
   set height [expr $width*0.6]
   set fill $Display_Node_Fill(Resource)
   set extra [extra_tag $canvas $etag]

   set main [$canvas create rectangle \
    [expr $x+$width] [expr $y+$height] \
    [expr $x-$width] [expr $y-$height] -width 2 \
            -outline $color -fill $fill -tag \
             "[get_resource_node_tags $tag] $extra"]

### changed by mike: _ is changed to line break
    set listext [split $text "|"]
    set text3 [join $listext "\n"]
    set t \
     [$canvas create text \
      $x $y -text "$text3" -width [expr $width*1.8] -font $fn\
      -tag "[get_resource_label_tags $tag] $extra"]

   lappend Display_Nodes($canvas,Resource) $main
   set Display_Node_Tag($canvas,$main) $tag
   set Display_Node_Main($canvas,$tag) $main

   set Display_Node_Pos($canvas,$tag) "$x $y"

   set Display_Node_Text($canvas,$tag) "$t $text"
   set Display_Node_ID($canvas,$text) $tag
   set Display_Node_Context($canvas,$tag) "$etag"

   if {$drag == 1} {
       bind_node $canvas $tag Resource
   }

   if {$extra != ""} {
       foreach i $extra {
         adjust_context_region $canvas $i
       }
   }


   check_adjust_canvas $canvas $x $y
}

############################################################################
# Task Nodes
############################################################################  

proc get_task_node_tags {tag} {
   return "[get_node_tags $tag] Task"
}

proc get_task_label_tags {tag} {
   return "[get_label_tags $tag] Task"
}

proc get_task_misc_tags {tag} {
   return "[get_misc_tags $tag] Task"
}

proc create_task_node {canvas x y tag {label ""} {drag 1} {etag ""}} {
   global Display_Symbology Display_Size Display_Node_Fill Display_Node_Pos 
   global Display_Nodes Display_Node_Tag Display_Node_Text Display_Node_ID
   global Display_Node_Context Display_Font Font_Type Display_Node_Main
   global Display_Node_Line Object_No Display_Font_Tag

###added by mike
    if {[info exists Display_Font_Tag($tag)]} {
	set fn $Display_Font_Tag($tag)
    } else {
	set fn $Display_Font(Node,Task)
    }
# puts $tag
# puts $fn
 

   if {$label == ""} {
       set text $tag
   } else {
       set text $label
   }

    if {$Object_No < 5} {
	set number $Object_No
    } else {
	set number [expr fmod($Object_No,5)]
    }
    set x [expr $x + $number*10]
    set y [expr $y +$number*10]

   set color $Display_Symbology(Task)
   set width $Display_Size(Task)
   set height [expr $width*0.6]
   set tilt [expr $width*0.3]
   set fill $Display_Node_Fill(Task)  
   set extra [extra_tag $canvas $etag]

   set main [$canvas create polygon [expr $x-$width] $y \
    [expr $x-$width+$tilt] [expr $y-$height] \
    [expr $x+$width-$tilt] [expr $y-$height] \
    [expr $x+$width] $y \
    [expr $x+$width-$tilt] [expr $y+$height] \
    [expr $x-$width+$tilt] [expr $y+$height] \
    [expr $x-$width] $y \
          -fill $fill -tag \
          "[get_task_node_tags $tag] $extra"]

   set line [$canvas create line [expr $x-$width] $y \
    [expr $x-$width+$tilt] [expr $y-$height] \
    [expr $x+$width-$tilt] [expr $y-$height] \
    [expr $x+$width] $y \
    [expr $x+$width-$tilt] [expr $y+$height] \
    [expr $x-$width+$tilt] [expr $y+$height] \
    [expr $x-$width] $y \
          -fill $color -width 2 -tag \
          "[get_task_misc_tags $tag] $extra"]

### changed by mike: _ is changed to line break
    set listext [split $text "|"]
    set text3 [join $listext "\n"]
    set t \
     [$canvas create text \
      $x $y -text "$text3" -width [expr $width*1.8] -font $fn\
      -tag "[get_task_label_tags $tag] $extra"]

   lappend Display_Nodes($canvas,Task) $main
   set Display_Node_Tag($canvas,$main) $tag
   set Display_Node_Main($canvas,$tag) $main
   set Display_Node_Line($canvas,$tag) $line
#    puts $main...$tag...$main....$line
   set Display_Node_Pos($canvas,$tag) "$x $y"
#    puts  $Display_Node_Pos($canvas,$tag)
   set Display_Node_Text($canvas,$tag) "$t $text"
#    puts $Display_Node_Text($canvas,$tag) 
   set Display_Node_ID($canvas,$text) $tag
#    puts $Display_Node_ID($canvas,$text) 
   set Display_Node_Context($canvas,$tag) "$etag"
#    puts $Display_Node_Context($canvas,$tag) 
   if {$drag == 1} {
       bind_node $canvas $tag Task
   }

   if {$extra != ""} {
       foreach i $extra {
         adjust_context_region $canvas $i
       }
   }

   check_adjust_canvas $canvas $x $y
}  

############################################################################
# Goal Nodes
############################################################################  

proc get_goal_node_tags {tag} {
   return "[get_node_tags $tag] Goal"
}

proc get_goal_label_tags {tag} {
   return "[get_label_tags $tag] Goal"
}

proc get_goal_misc_tags {tag} {
   return "[get_misc_tags $tag] Goal"
}

proc create_goal_node {canvas x y tag {label ""} {drag 1} {etag ""}} {
   global Display_Symbology Display_Size 
   global Display_Node_Fill Display_Node_Pos Display_Nodes
   global Display_Node_Text Display_Node_Tag Display_Node_ID
   global Display_Node_Context
   global Display_Font Font_Type Display_Font_Tag
   global Display_Node_Main Display_Node_Line Object_No

###added by mike
    if {[info exists Display_Font_Tag($tag)]} {
	set fn $Display_Font_Tag($tag)
    } else {
	set fn $Display_Font(Node,Goal)
    }
# puts $tag
# puts $fn

   if {$label == ""} {
       set text $tag
   } else {
       set text $label
   }

    if {$Object_No < 5} {
	set number $Object_No
    } else {
	set number [expr fmod($Object_No,5)]
    }
    set x [expr $x + $number*10]
    set y [expr $y +$number*10]

   set color $Display_Symbology(Goal)  
   set width $Display_Size(Goal)
   set height [expr $width*0.6]
   set tilt [expr $width*0.3]
   set fill $Display_Node_Fill(Goal)
   set extra [extra_tag $canvas $etag]
### the old version   
#   set main [$canvas create polygon [expr $x-$width] $y \
    [expr $x-$width+$tilt] [expr $y-$height] \
    [expr $x+$width-$tilt] [expr $y-$height] \
    [expr $x+$width] $y \
    [expr $x+$width-$tilt] [expr $y+$height] \
    [expr $x-$width+$tilt] [expr $y+$height] \
    [expr $x-$width] $y \
          -smooth 1 -fill $fill -tag \
          "[get_goal_node_tags $tag] $extra"]

#   set line [$canvas create line [expr $x-$width] $y \
    [expr $x-$width+$tilt] [expr $y-$height] \
    [expr $x+$width-$tilt] [expr $y-$height] \
    [expr $x+$width] $y \
    [expr $x+$width-$tilt] [expr $y+$height] \
    [expr $x-$width+$tilt] [expr $y+$height] \
    [expr $x-$width] $y \
          -smooth 1 -fill $color -width 2 -tag \
          "[get_goal_misc_tags $tag] $extra"]
### the new version by mike
set xcoord 0
set ycoord 0
set main [$canvas create polygon [expr $xcoord+$x-$width-3] [expr $ycoord+$y] \
    [expr $xcoord+$x-$width+$tilt-12] [expr $ycoord+$y-$height+11] \
    [expr $xcoord+$x-$width+$tilt-5] [expr $ycoord+$y-$height+3] \
    [expr $xcoord+$x-$width+$tilt-1] [expr $ycoord+$y-$height] \
    [expr $xcoord+$x-$width+$tilt] [expr $ycoord+$y-$height] \
    [expr $xcoord+$x+$width-$tilt] [expr $ycoord+$y-$height] \
    [expr $xcoord+$x+$width-$tilt+1] [expr $ycoord+$y-$height] \
    [expr $xcoord+$x+$width-$tilt+5] [expr $ycoord+$y-$height+3] \
    [expr $xcoord+$x+$width-$tilt+12] [expr $ycoord+$y-$height+11] \
    [expr $xcoord+$x+$width+3] [expr $ycoord+$y] \
    [expr $xcoord+$x+$width-$tilt+12] [expr $ycoord+$y+$height-11] \
    [expr $xcoord+$x+$width-$tilt+5] [expr $ycoord+$y+$height-3] \
    [expr $xcoord+$x+$width-$tilt+1] [expr $ycoord+$y+$height] \
    [expr $xcoord+$x+$width-$tilt] [expr $ycoord+$y+$height] \
    [expr $xcoord+$x-$width+$tilt] [expr $ycoord+$y+$height] \
    [expr $xcoord+$x-$width+$tilt-1] [expr $ycoord+$y+$height] \
    [expr $xcoord+$x-$width+$tilt-5] [expr $ycoord+$y+$height-3] \
    [expr $xcoord+$x-$width+$tilt-12] [expr $ycoord+$y+$height-11] \
	      [expr $xcoord+$x-$width-3] [expr $ycoord+$y] \
          -smooth 1 -fill $fill -tag \
          "[get_goal_node_tags $tag] $extra"]

set line [$canvas create line [expr $xcoord+$x-$width-3] [expr $ycoord+$y] \
    [expr $xcoord+$x-$width+$tilt-12] [expr $ycoord+$y-$height+11] \
    [expr $xcoord+$x-$width+$tilt-5] [expr $ycoord+$y-$height+3] \
    [expr $xcoord+$x-$width+$tilt-1] [expr $ycoord+$y-$height] \
    [expr $xcoord+$x-$width+$tilt] [expr $ycoord+$y-$height] \
    [expr $xcoord+$x+$width-$tilt] [expr $ycoord+$y-$height] \
    [expr $xcoord+$x+$width-$tilt+1] [expr $ycoord+$y-$height] \
    [expr $xcoord+$x+$width-$tilt+5] [expr $ycoord+$y-$height+3] \
    [expr $xcoord+$x+$width-$tilt+12] [expr $ycoord+$y-$height+11] \
    [expr $xcoord+$x+$width+3] [expr $ycoord+$y] \
    [expr $xcoord+$x+$width-$tilt+12] [expr $ycoord+$y+$height-11] \
    [expr $xcoord+$x+$width-$tilt+5] [expr $ycoord+$y+$height-3] \
    [expr $xcoord+$x+$width-$tilt+1] [expr $ycoord+$y+$height] \
    [expr $xcoord+$x+$width-$tilt] [expr $ycoord+$y+$height] \
    [expr $xcoord+$x-$width+$tilt] [expr $ycoord+$y+$height] \
    [expr $xcoord+$x-$width+$tilt-1] [expr $ycoord+$y+$height] \
    [expr $xcoord+$x-$width+$tilt-5] [expr $ycoord+$y+$height-3] \
    [expr $xcoord+$x-$width+$tilt-12] [expr $ycoord+$y+$height-11] \
	      [expr $xcoord+$x-$width-3] [expr $ycoord+$y] \
            -smooth 1 -fill $color -width 2 -tag \
          "[get_goal_misc_tags $tag] $extra"]

### changed by mike: _ is changed to line break
set listext [split $text "|"]
    set text3 [join $listext "\n"]
    set t \
     [$canvas create text \
      $x $y -text "$text3" -width [expr $width*1.8] -font $fn\
      -tag "[get_goal_label_tags $tag] $extra"]

   lappend Display_Nodes($canvas,Goal) $main
   set Display_Node_Tag($canvas,$main) $tag
   set Display_Node_Main($canvas,$tag) $main
   set Display_Node_Line($canvas,$tag) $line

   set Display_Node_Pos($canvas,$tag) "$x $y"

   set Display_Node_Text($canvas,$tag) "$t $text"
   set Display_Node_ID($canvas,$text) $tag

   set Display_Node_Context($canvas,$tag) $etag

   if {$drag == 1} {
       bind_node $canvas $tag Goal
   }

   if {$extra != ""} {
       foreach i $extra {
         adjust_context_region $canvas $i
       }
   }

   check_adjust_canvas $canvas $x $y
}

############################################################################
# Softgoal Nodes
############################################################################  

proc get_softgoal_node_tags {tag} {
   return "[get_node_tags $tag] SGoal"
}

proc get_softgoal_label_tags {tag} {
   return "[get_label_tags $tag] SGoal"
}

proc get_softgoal_misc_tags {tag} {
   return "[get_misc_tags $tag] SGoal"
}


 
proc create_softgoal_node {canvas x y tag {label ""} {drag 1} {etag ""}} {
   global Display_Symbology Display_Size 
   global Display_Node_Fill Display_Node_Pos Display_Nodes
   global Display_Node_Text Display_Node_Tag Display_Node_ID
   global Display_Node_Context Display_Node_Line
   global Display_Font Font_Type
   global Display_Node_Main
   global Display_Font_Tag  Object_No

###added by mike

    if {[info exists Display_Font_Tag($tag)]} {
#	puts $tag,mmmmmmmmm
	set fn $Display_Font_Tag($tag)
    } else {
	set fn $Display_Font(Node,Softgoal)
    }
# puts $tag
# puts $fn

   if {$label == ""} {
       set text $tag
   } else {
       set text $label
   }

    if {$Object_No < 5} {
	set number $Object_No
    } else {
	set number [expr fmod($Object_No,5)]
    }
    set x [expr $x + $number*10]
    set y [expr $y +$number*10]

   set color $Display_Symbology(Softgoal)  
   set width [expr $Display_Size(Softgoal)*1.0]
   set height [expr $width*0.6]
   set tilt [expr $width*0.4]
   set fill $Display_Node_Fill(Softgoal)
   set extra [extra_tag $canvas $etag]
 
   set main [$canvas create polygon [expr $x-$width] [expr $y-$height-$tilt] \
           $x [expr $y-$tilt*1.6] \
           [expr $x+$width+$tilt] [expr $y-$height] \
           [expr $x+$width] [expr $y+$height+$tilt] \
           $x [expr $y+$tilt*1.6] \
           [expr $x-$width-$tilt] [expr $y+$height] \
           [expr $x-$width] [expr $y-$height-$tilt] \
           -fill $fill -smooth 1 -tag \
           "[get_softgoal_node_tags $tag] $extra"]

   set line [$canvas create line [expr $x-$width] [expr $y-$height-$tilt] \
           $x [expr $y-$tilt*1.6] \
           [expr $x+$width+$tilt] [expr $y-$height] \
           [expr $x+$width] [expr $y+$height+$tilt] \
           $x [expr $y+$tilt*1.6] \
           [expr $x-$width-$tilt] [expr $y+$height] \
           [expr $x-$width] [expr $y-$height-$tilt] \
           -fill $color -smooth 1 -width 2 -tag \
           "[get_softgoal_misc_tags $tag] $extra"]

### changed by mike: _ is changed to line break
    set listext [split $text "|"]
    set text3 [join $listext "\n"]
    set t \
     [$canvas create text \
      $x $y -text "$text3" -width [expr $width*1.8] -font $fn\
      -tag "[get_softgoal_label_tags $tag] $extra"]

   lappend Display_Nodes($canvas,Softgoal) $main
   set Display_Node_Tag($canvas,$main) $tag
   set Display_Node_Main($canvas,$tag) $main
   set Display_Node_Line($canvas,$tag) $line    
#   set Display_Font_Tag($canvas,$tag) $Display_Font(Node,Softgoal) 
   set Display_Node_Pos($canvas,$tag) "$x $y"

   set Display_Node_Text($canvas,$tag) "$t $text"
   set Display_Node_ID($canvas,$text) $tag

   set Display_Node_Context($canvas,$tag) $etag

   if {$drag == 1} {
       bind_node $canvas $tag Softgoal
   }

   if {$extra != ""} {
       foreach i $extra {
         adjust_context_region $canvas $i
       }
   }

   check_adjust_canvas $canvas $x $y
}


